#include "test/Expectation.h"

#include <nlohmann/json.hpp>
#include <stdexcept>


namespace xs { namespace test
{
    Expectation::Expectation(const nlohmann::json& object)
    : name_(object["command"].get<std::string>())
    {
        request_ = object["param"];

        const auto& expect = object["expect"];
        response_ = expect["response"];

        if (expect.count("runs")) {
            for (const auto& element : expect["runs"]) {
                if (!element.is_string()) {
                    throw std::runtime_error("element in \"runs\" is not string");
                }
                runs_.emplace(element.get<std::string>());
            }
        }

        if (expect.count("posts")) {
            for (const auto& element : expect["posts"]) {
                if (!element.is_string()) {
                    throw std::runtime_error("element in \"posts\" is not string");
                }
                posts_.emplace(element.get<std::string>());
            }
        }
    }

    bool Expectation::verify(const nlohmann::json& object) noexcept
    {
        if (!object.is_object()) {
            return false;
        }

        if (!(object.count("command") && object["command"].is_string())) {
            return false;
        }

        if (!object.count("param")) {
            return false;
        }

        if (!(object["param"].is_string() || object["param"].is_object())) {
            return false;
        }

        if (!(object.count("expect") && object["expect"].is_object())) {
            return false;
        }

        const auto& expect = object["expect"];
        if (!expect.count("response")) {
            return false;
        }

        if (!(expect["response"].is_string() || expect["response"].is_object())) {
            return false;
        }

        if (expect.count("runs") && !expect["runs"].is_array()) {
            return false;
        }

        if (expect.count("posts") && !expect["posts"].is_array()) {
            return false;
        }

        return true;
    }
} }
